/* **********************************************************
 * Copyright (C) 1998-2000 VMware, Inc.
 * All Rights Reserved
 * $Id: vm_time.h,v 1.4 2000/06/03 02:32:51 yoel Exp $
 * **********************************************************/


/*
 * vm_time.h  --
 *
 *    Time management functions.
 *    Part of driver-only distribution
 *
 *    see comment in poll.c
 */


#ifndef VM_TIME_H
#define VM_TIME_H


#include "vm_types.h"

/* TS stands for "timestamp", which is in units of "cycles" */
/* Use these types to express time, Get_TSC should return a
   VmAbsoluteTS -- converting to US results in a VmAbsoluteUS.
   Subtracting two VmAbsoluteTS's results in a VmRelativeTS, likewise
   for VmAbsoluteUS and VmRelativeUS */
/* I use these instead of VmTimeRealClock and VmTimeVirtualClock
   because those types are not used consistently in our code (cycles vs
   us) */
typedef uint64 VmAbsoluteTS; //a particular point in time (in cycles)
typedef int64 VmRelativeTS; //a signed delta in cycles
typedef uint64 VmAbsoluteUS; //a particular point in time (in us)
typedef int64 VmRelativeUS; //a signed delta in us

struct VmTimeVirtualRealClock;
typedef struct VmTimeVirtualRealClock VmTimeVirtualRealClock;

#define VMTIME_VIRTUAL_INFINITE  ((((VmTimeVirtualClock)1)<<62)-1)

#define CYCLES_TO_USECS(vm, _c)  ((_c) / (vm)->miscShared->mhzEstimate)
#define USECS_TO_CYCLES(vm, _us) ((_us) * (vm)->miscShared->mhzEstimate)

#ifdef MONITOR
#define GET_REALCYCLES()         (GetTSC())
#define GET_REALTIME_IN_USECS(vm) (CYCLES_TO_USECS(vm, GET_REALCYCLES()))
#else
#define GET_REALCYCLES()         GetTSC()
#define GET_REALTIME_IN_USECS(vm) (Hostinfo_FastRealTime() / \
				  (Hostinfo_FastRealTimeFrequency() / 1000000))
#endif

#ifdef USERLEVEL
struct VM;
extern VmTimeType VmTime_ReadVirtualTime(void);
extern void VmTime_IncVirtualTime(struct VM *vm, VmTimeType amount);
extern VmTimeVirtualRealClock *VmTime_NewVirtualRealClock(struct VM *);
extern void VmTime_StartVirtualRealClock(VmTimeVirtualRealClock *, double);
extern void VmTime_ResetVirtualRealClock(VmTimeVirtualRealClock *);
extern VmTimeType VmTime_ReadVirtualRealTime(VmTimeVirtualRealClock *);
extern VmTimeType VmTime_RemainingVirtualRealTime(VmTimeVirtualRealClock *,
						  VmTimeType realTime);
extern void VmTime_UpdateVirtualRealTime(VmTimeVirtualRealClock *clock,
                                         VmTimeType realTime,
                                         VmTimeType virtualTime);
#endif
#endif /* VM_TIME_H */

